#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "ladspa.h"

#define KARAKOE_GAIN                   0
#define KARAKOE_LIN                    1
#define KARAKOE_RIN                    2
#define KARAKOE_LOUT                   3
#define KARAKOE_ROUT                   4

static LADSPA_Descriptor *karakoeDescriptor = NULL;

typedef struct {
	LADSPA_Data *gain;
	LADSPA_Data *lin;
	LADSPA_Data *rin;
	LADSPA_Data *lout;
	LADSPA_Data *rout;
	LADSPA_Data run_adding_gain;
} Karakoe;

const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {
	switch (index) {
	case 0:
		return karakoeDescriptor;
	default:
		return NULL;
	}
}

static void cleanupKarakoe(LADSPA_Handle instance) {
	free(instance);
}

static void connectPortKarakoe(
 LADSPA_Handle instance,
 unsigned long port,
 LADSPA_Data *data) {
	Karakoe *plugin;

	plugin = (Karakoe *)instance;
	switch (port) {
	case KARAKOE_GAIN:
		plugin->gain = data;
		break;
	case KARAKOE_LIN:
		plugin->lin = data;
		break;
	case KARAKOE_RIN:
		plugin->rin = data;
		break;
	case KARAKOE_LOUT:
		plugin->lout = data;
		break;
	case KARAKOE_ROUT:
		plugin->rout = data;
		break;
	}
}

static LADSPA_Handle instantiateKarakoe(
 const LADSPA_Descriptor *descriptor,
 unsigned long s_rate) {
	Karakoe *plugin_data = (Karakoe *)malloc(sizeof(Karakoe));
	plugin_data->run_adding_gain = 1.0f;

	return (LADSPA_Handle)plugin_data;
}

#undef buffer_write
#undef RUN_ADDING
#undef RUN_REPLACING

#define buffer_write(b, v) (b = v)
#define RUN_ADDING    0
#define RUN_REPLACING 1

static void runKarakoe(LADSPA_Handle instance, unsigned long sample_count) {
	Karakoe *plugin_data = (Karakoe *)instance;

	/* Vocal volume (dB) (float value) */
	const LADSPA_Data gain = *(plugin_data->gain);

	/* Left in (array of floats of length sample_count) */
	const LADSPA_Data * const lin = plugin_data->lin;

	/* Right in (array of floats of length sample_count) */
	const LADSPA_Data * const rin = plugin_data->rin;

	/* Left out (array of floats of length sample_count) */
	LADSPA_Data * const lout = plugin_data->lout;

	/* Right out (array of floats of length sample_count) */
	LADSPA_Data * const rout = plugin_data->rout;

#line 17 "karaoke_1409.xml"
	unsigned long pos;
	float coef = pow(10.0f, gain * 0.05f) * 0.5f;
	float m, s;

	for (pos = 0; pos < sample_count; pos++) {
	  m = lin[pos] + rin[pos];
	  s = lin[pos] - rin[pos];
	  buffer_write(lout[pos], m * coef + s * 0.5f);
	  buffer_write(rout[pos], m * coef - s * 0.5f);
	}
}
#undef buffer_write
#undef RUN_ADDING
#undef RUN_REPLACING

#define buffer_write(b, v) (b += (v) * run_adding_gain)
#define RUN_ADDING    1
#define RUN_REPLACING 0

static void setRunAddingGainKarakoe(LADSPA_Handle instance, LADSPA_Data gain) {
	((Karakoe *)instance)->run_adding_gain = gain;
}

static void runAddingKarakoe(LADSPA_Handle instance, unsigned long sample_count) {
	Karakoe *plugin_data = (Karakoe *)instance;
	LADSPA_Data run_adding_gain = plugin_data->run_adding_gain;

	/* Vocal volume (dB) (float value) */
	const LADSPA_Data gain = *(plugin_data->gain);

	/* Left in (array of floats of length sample_count) */
	const LADSPA_Data * const lin = plugin_data->lin;

	/* Right in (array of floats of length sample_count) */
	const LADSPA_Data * const rin = plugin_data->rin;

	/* Left out (array of floats of length sample_count) */
	LADSPA_Data * const lout = plugin_data->lout;

	/* Right out (array of floats of length sample_count) */
	LADSPA_Data * const rout = plugin_data->rout;

#line 17 "karaoke_1409.xml"
	unsigned long pos;
	float coef = pow(10.0f, gain * 0.05f) * 0.5f;
	float m, s;

	for (pos = 0; pos < sample_count; pos++) {
	  m = lin[pos] + rin[pos];
	  s = lin[pos] - rin[pos];
	  buffer_write(lout[pos], m * coef + s * 0.5f);
	  buffer_write(rout[pos], m * coef - s * 0.5f);
	}
}

void _init() {
	char **port_names;
	LADSPA_PortDescriptor *port_descriptors;
	LADSPA_PortRangeHint *port_range_hints;

	karakoeDescriptor =
	 (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));

	if (karakoeDescriptor) {
		karakoeDescriptor->UniqueID = 1409;
		karakoeDescriptor->Label = strdup("karakoe");
		karakoeDescriptor->Properties =
		 LADSPA_PROPERTY_HARD_RT_CAPABLE;
		karakoeDescriptor->Name =
		 strdup("Karaoke");
		karakoeDescriptor->Maker =
		 strdup("Steve Harris <steve@plugin.org.uk>");
		karakoeDescriptor->Copyright =
		 strdup("GPL");
		karakoeDescriptor->PortCount = 5;

		port_descriptors = (LADSPA_PortDescriptor *)calloc(5,
		 sizeof(LADSPA_PortDescriptor));
		karakoeDescriptor->PortDescriptors =
		 (const LADSPA_PortDescriptor *)port_descriptors;

		port_range_hints = (LADSPA_PortRangeHint *)calloc(5,
		 sizeof(LADSPA_PortRangeHint));
		karakoeDescriptor->PortRangeHints =
		 (const LADSPA_PortRangeHint *)port_range_hints;

		port_names = (char **)calloc(5, sizeof(char*));
		karakoeDescriptor->PortNames =
		 (const char **)port_names;

		/* Parameters for Vocal volume (dB) */
		port_descriptors[KARAKOE_GAIN] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
		port_names[KARAKOE_GAIN] =
		 strdup("Vocal volume (dB)");
		port_range_hints[KARAKOE_GAIN].HintDescriptor =
		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
		port_range_hints[KARAKOE_GAIN].LowerBound = -70;
		port_range_hints[KARAKOE_GAIN].UpperBound = 0;

		/* Parameters for Left in */
		port_descriptors[KARAKOE_LIN] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
		port_names[KARAKOE_LIN] =
		 strdup("Left in");
		port_range_hints[KARAKOE_LIN].HintDescriptor = 0;

		/* Parameters for Right in */
		port_descriptors[KARAKOE_RIN] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
		port_names[KARAKOE_RIN] =
		 strdup("Right in");
		port_range_hints[KARAKOE_RIN].HintDescriptor = 0;

		/* Parameters for Left out */
		port_descriptors[KARAKOE_LOUT] =
		 LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
		port_names[KARAKOE_LOUT] =
		 strdup("Left out");
		port_range_hints[KARAKOE_LOUT].HintDescriptor = 0;

		/* Parameters for Right out */
		port_descriptors[KARAKOE_ROUT] =
		 LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
		port_names[KARAKOE_ROUT] =
		 strdup("Right out");
		port_range_hints[KARAKOE_ROUT].HintDescriptor = 0;

		karakoeDescriptor->activate = NULL;
		karakoeDescriptor->cleanup = cleanupKarakoe;
		karakoeDescriptor->connect_port = connectPortKarakoe;
		karakoeDescriptor->deactivate = NULL;
		karakoeDescriptor->instantiate = instantiateKarakoe;
		karakoeDescriptor->run = runKarakoe;
		karakoeDescriptor->run_adding = runAddingKarakoe;
		karakoeDescriptor->set_run_adding_gain = setRunAddingGainKarakoe;
	}
}

void _fini() {
	int i;

	if (karakoeDescriptor) {
		free((char *)karakoeDescriptor->Label);
		free((char *)karakoeDescriptor->Name);
		free((char *)karakoeDescriptor->Maker);
		free((char *)karakoeDescriptor->Copyright);
		free((LADSPA_PortDescriptor *)karakoeDescriptor->PortDescriptors);
		for (i = 0; i < karakoeDescriptor->PortCount; i++)
			free((char *)(karakoeDescriptor->PortNames[i]));
		free((char **)karakoeDescriptor->PortNames);
		free((LADSPA_PortRangeHint *)karakoeDescriptor->PortRangeHints);
		free(karakoeDescriptor);
	}

}
